
from django.views import generic

from mobile_views import utilities

__all__ = (
    'MobileTemplateView', 'MobileDetailView', 'MobileFormView',
    'MobileCreateView', 'MobileRedirectView',
)

class MobileMixin(object):
    """
    MobileMixin is a supplement to django's generic views.

    The mobile mixin uses the user agent to match a regular expression
    that is know to indicate a mobile device.

    If the device is mobile and the user doesn't have a 'no_mobile'
    cookie set, the Mixin will try to render the mobile template if
    it exists, falling back to the regular template if not.

    """
    template_name = None
    mobile_template_name = None

    def use_mobile(self):
        return utilities.use_mobile(self.request)

    def set_mobile_cookie(self, response):
        response.set_cookie('no_mobile', True)

    def delete_mobile_cookie(self, response):
        response.delete_cookie('no_mobile')

    def get_template_names(self):
        """
        Returns a list of template names to be used for the request. Must return
        a list. May not be called if render_to_response is overridden.
        """
        if self.use_mobile():
            return self.get_mobile_template_name()
        else:
            return self.get_default_template_name()

    def get_default_template_name(self):
        if self.template_name is None:
            return []
        return [self.template_name]

    def get_mobile_template_name(self):
        if self.mobile_template_name is None:
            return self.get_default_template_name()
        return [self.mobile_template_name]


# The views below just piggyback on django's generic views
# adding the mobile mixin...

class MobileTemplateView(MobileMixin, generic.TemplateView):
    pass

class MobileDetailView(MobileMixin, generic.DetailView):
    pass

class MobileFormView(MobileMixin, generic.FormView):
    pass

class MobileCreateView(MobileMixin, generic.CreateView):
    pass

class MobileRedirectView(MobileMixin, generic.RedirectView):
    permanent = False


__author__ = 'amadison'

from mobile_views.generic_views import MobileRedirectView


__all__ = ('FullSiteView', )

class FullSiteView(MobileRedirectView):
    """
    this view will set a 'no_mobile' cookie and redirect to whatever
    the follow url is. The cookie will remain for the session of the
    browser and prevent any further re-directing to a mobile template.
    """

    def get_redirect_url(self, **kwargs):
        return self.request.GET.get('follow', '/')

    def get(self, request, *args, **kwargs):
        response = super(FullSiteView, self).get(request, *args, **kwargs)
        self.set_mobile_cookie(response)
        return response




import mock

from django import test
from django.http import HttpResponse, HttpRequest

from mobile_views import views, utilities, generic_views

class MobileMixinTests(test.TestCase):

    def setUp(self):
        self.request = mock.Mock(spec=HttpRequest)
        self.request.META = {}
        self.mixin = generic_views.MobileMixin()
        self.mixin.request = self.request

    def test_sets_no_mobile_cookie(self):
        response = mock.Mock(spec=HttpResponse)
        self.mixin.set_mobile_cookie(response)
        self.assertEqual((('no_mobile', True), {}), response.set_cookie.call_args)

    def test_deletes_no_mobile_cookie(self):
        response = mock.Mock(spec=HttpResponse)
        self.mixin.delete_mobile_cookie(response)
        self.assertEqual((('no_mobile',), {}), response.delete_cookie.call_args)

    @mock.patch('mobile_views.utilities.use_mobile', mock.Mock(return_value=True))
    def test_return_true_when_utilities_function_uses_mobile(self):
        self.assertEqual(True, self.mixin.use_mobile())

    @mock.patch('mobile_views.utilities.use_mobile', mock.Mock(return_value=False))
    def test_return_true_when_utilities_function_uses_mobile(self):
        self.assertEqual(False, self.mixin.use_mobile())

    @mock.patch('mobile_views.generic_views.MobileMixin.use_mobile', mock.Mock(return_value=True))
    def test_returns_mobile_template_name_when_use_mobile(self):
        self.mixin.mobile_template_name = mock.sentinel.mobile_template_name
        self.assertEqual([self.mixin.mobile_template_name], self.mixin.get_template_names())

    @mock.patch('mobile_views.generic_views.MobileMixin.use_mobile', mock.Mock(return_value=False))
    def test_returns_template_name_when_not_use_mobile(self):
        self.mixin.template_name = mock.sentinel.template_name
        self.assertEqual([self.mixin.template_name], self.mixin.get_template_names())

    @mock.patch('mobile_views.generic_views.MobileMixin.use_mobile', mock.Mock(return_value=False))
    def test_returns_empty_list_when_not_use_mobile_and_template_name_not_defined(self):
        self.assertEqual([], self.mixin.get_template_names())

class MobileUtilitiesTests(test.TestCase):

    def test_returns_true_when_user_agent_matches_android(self):
        request = mock.Mock(spec=HttpRequest)
        request.META = {'HTTP_USER_AGENT': 'android'}
        self.assertTrue(utilities.is_mobile(request))

    def test_returns_true_when_user_agent_matches_iphone(self):
        request = mock.Mock(spec=HttpRequest)
        request.META = {'HTTP_USER_AGENT': 'iphone'}
        self.assertTrue(utilities.is_mobile(request))

    def test_returns_false_when_user_agent_does_not_match_lists(self):
        request = mock.Mock(spec=HttpRequest)
        request.META = {'HTTP_USER_AGENT': 'chrome'}
        self.assertFalse(utilities.is_mobile(request))

    def test_returns_true_when_user_has_no_mobile_cookie(self):
        request = mock.Mock(spec=HttpRequest)
        request.COOKIES = {'no_mobile': 'True'}
        self.assertTrue(utilities.user_declined_mobile(request))

    def test_returns_false_when_user_does_not_have_no_mobile_cookie(self):
        request = mock.Mock(spec=HttpRequest)
        request.COOKIES = {}
        self.assertFalse(utilities.user_declined_mobile(request))

    @mock.patch('mobile_views.utilities.is_mobile', mock.Mock(return_value=True))
    @mock.patch('mobile_views.utilities.user_declined_mobile', mock.Mock(return_value=False))
    def test_returns_true_when_is_mobile_and_not_user_declined_mobile(self):
        self.assertEqual(True, utilities.use_mobile(mock.Mock()))

    @mock.patch('mobile_views.utilities.is_mobile', mock.Mock(return_value=False))
    @mock.patch('mobile_views.utilities.user_declined_mobile', mock.Mock(return_value=False))
    def test_returns_false_when_not_is_mobile(self):
        self.assertEqual(False, utilities.use_mobile(mock.Mock()))

    @mock.patch('mobile_views.utilities.is_mobile', mock.Mock(return_value=True))
    @mock.patch('mobile_views.utilities.user_declined_mobile', mock.Mock(return_value=True))
    def test_returns_false_when_is_mobile_and_user_declined_mobile(self):
        self.assertEqual(False, utilities.use_mobile(mock.Mock()))

class FullSiteViewTests(test.TestCase):

    def setUp(self):
        self.view = views.FullSiteView()

    def test_returns_follow_parameter(self):
        self.view.request = mock.Mock()
        self.view.request.GET = {'follow': mock.sentinel.follow}
        self.assertEqual(mock.sentinel.follow, self.view.get_redirect_url())

    def test_returns_root_url_when_no_follow_parameter(self):
        self.view.request = mock.Mock()
        self.view.request.GET = {}
        self.assertEqual('/', self.view.get_redirect_url())

    @mock.patch('mobile_views.views.FullSiteView.set_mobile_cookie')
    @mock.patch('mobile_views.views.MobileRedirectView.get')
    def test_sets_mobile_cookie_on_get_request(self, redirect_get, set_mobile_cookie):
        response = mock.Mock(spec=HttpResponse)
        redirect_get.return_value = response
        views.FullSiteView().get(mock.Mock())
        self.assertTrue(((response,), {}), set_mobile_cookie.call_args)




from setuptools import setup, find_packages

REQUIREMENTS = (
    'django>=1.3',
)
TEST_REQUIREMENTS = (
    'mock',
)

from mobile_views import VERSION

setup(
    name="django-mobile-views",
    version=VERSION,
    author="Aaron Madison",
    description="Some mixins to serve mobile templates when mobile detected.",
    long_description=open('README', 'r').read(),
    url="https://github.com/madisona/django-mobile-views",
    packages=find_packages(exclude=["example"]),
    install_requires=REQUIREMENTS,
    tests_require=TEST_REQUIREMENTS,
    zip_safe=False,
    classifiers = [
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        ],
    )